/*
 * Copyright (c) 2021 «FORS Telecom» LLC.
 * All Rights Reserved.
 *
 * The code is published under the terms of the «FORS Telecom» LLC Licens,
 * published in file LICENSE of this package or at
 *
 * http://lui.fors.ru/license	- in Russian
 * http://lui.fors.com/license	- in English.
 *
 */

package LuiDb;

import LuiDbInterface.NeedMaintenance;
import java.util.ArrayList;
import java.util.Set;
import java.util.WeakHashMap;

/**
 * Обслуживание экземпляров NeedMaintenance. Вызывает метов maintenance примерно раз в 10 секунд.
 * Хранятся Weak-ссылки, поэтому ничего разрегистрировать не нужно.
 * @author geliseev
 */
public class Maintainer {
    
    
    /** Поток выполнения обслуживателя */
    private static Thread stMaintenanceThread = null;
    
    /** Weak-коллекция обслуживаемых объектов */
    private static final WeakHashMap<NeedMaintenance, Object> stEscorted = new WeakHashMap<>();
    
    /** Объект-заглушка */
    private static final Object stStub = new Object();
    
    
   
    /**
     * Вызывается при завершении работы потока обслуживания
     */
    private static synchronized void threadEnds() {
        stMaintenanceThread = null;
    }
    
    /**
     * Создание потока обслуживания, если его ещё нет
     */
    private static synchronized void runThread() {
        
        if( stMaintenanceThread != null
            && stMaintenanceThread.isAlive()
           ) { return; }
        
        stMaintenanceThread = new Thread() {
            /**
             * Главная процедура потока обслуживания.
             * Автоматически завершается, если обслуживать нечего.
             */
            @Override
            public void run() {
                long aWait;
                long aLastStart;
                while(true) {
                    ArrayList<NeedMaintenance> aProc;
                    // Потокобезопасно
                    synchronized(stEscorted) {
                        // Скопируем всё, требующее обслуживания
                        aProc = new ArrayList<>(stEscorted.keySet());
                    }
                    
                    // Если набор пустой - то и крутить поток дальше не обязательно
                    if(aProc.isEmpty()) { break;}
                    
                    aLastStart = System.currentTimeMillis();
                    
                    // Вызов maintenance для всех требующих обслуживания
                    aProc.forEach((aEntry) -> {
                        try {
                            aEntry.maintenance();
                        }
                        catch(Throwable ignore) {}
                    });
                    
                    // Если не почистить - сборщик мусора не соберёт!
                    aProc.clear();
                    aProc=null;
                    
                    // Расчёт времени ожидания
                    // Пытаемся запустить каждый обработчик раз в 10 секунд,
                    // не зависимо от того, сколько времени обработчики работали...
                    aWait = 10000 - ( System.currentTimeMillis() - aLastStart );
                    
                    try {
                        // Какое-то время ждать нужно, иначе сборщик мусора 
                        // может очень долго не собирать уже ненужные объекты
                        Thread.sleep(Math.max(aWait, 1000));
                    } catch (InterruptedException ex) {
                        break;
                    }
                }
                threadEnds();
            }
        };
        
        stMaintenanceThread.setName("Common Maintenance");
        stMaintenanceThread.setDaemon(true);
        stMaintenanceThread.setUncaughtExceptionHandler((Thread t, Throwable e) -> {
            threadEnds();
        });
        stMaintenanceThread.start();
    }
    
    /**
     * Добавление обслуживаемого  объекта.
     * Если рабочего потока нет, то он создаётся прямо здесь
     * @param pEscort 
     */
    public static void add( NeedMaintenance pEscort )
    {
        synchronized(stEscorted) {
            stEscorted.put(pEscort, stStub);
        }
        runThread();
    }
    
    /**
     * Удаление объекта из желающих осблуживаться.
     * Реальное прекращеное обслуживангия может произойти несколько позже.
     * @param pEscort 
     */
    public static void remove( NeedMaintenance pEscort )
    {
        synchronized(stEscorted) {
            stEscorted.remove(pEscort);
        }
    }
    
}
