/*
 * Copyright (c) 2021 «FORS Telecom» LLC.
 * All Rights Reserved.
 *
 * The code is published under the terms of the «FORS Telecom» LLC Licens,
 * published in file LICENSE of this package or at
 *
 * http://lui.fors.ru/license	- in Russian
 * http://lui.fors.com/license	- in English.
 *
 */

/* 
 *   (С) 2016  ООО ФОРС-Центр Разработки, ООО ФОРС-Телеком
 *   http://www.fors.ru
 *   http://www.forstelecom.ru
 *
 *   Интерфес класса доступа к данным в СУБД 
 *   
 *   РЕВИЗИЯ:
 *  $Rev:: 1            $:  Ревизия файла
 *  $Author:: aliubushkin   $:  Автор последних изменений
 *  $Date:: 2017-11-14 1#$:  Дата последних изменений
 *   ПРИМЕЧАНИЯ:
****************************************************************************** */
package LuiDbInterface;

import LuiCollection.LuiCollection;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;


/**
 *
 * @author aliubushkin
 */
public interface LuiDbConn {
     /**
     * Соединение с БД
     * @param pDataBaseURL URL для доступа к БД например "jdbc:oracle:thin:@fst1:1521:fst" или ""jdbc:postgresql://localhost:5432/test"
     * @param pUsername имя пользователя
     * @param pPassword пароль
     * @param pSchema схема БД для "alter session set CURRENT_SCHEMA = СХЕМА " или для PG "set session search_path='СХЕМА'"
     * @param pAutocommit режим автокоммита
     * @param pReadonly режим только чтения
     * @throws SQLException
     */
public void connect(String pDataBaseURL,String pUsername,String pPassword,String pSchema,boolean pAutocommit,boolean pReadonly)throws SQLException;
    
    /**
     * Соединение с БД
     * @param pDataBaseURL URL для доступа к БД например "jdbc:oracle:thin:@fst1:1521:fst" или ""jdbc:postgresql://localhost:5432/test"
     * @param pUsername имя пользователя
     * @param pPassword пароль
     * @param pSchema схема
     * @throws SQLException
     */
public void connect(String pDataBaseURL,String pUsername,String pPassword,String pSchema)throws SQLException;

    /**
     * Отсоединение от БД
     * @throws SQLException
     */
public void disconnect()throws SQLException;


    /**
     * Выполнение DDL - операций
     * @param pStmt оператор SQL
     * @throws SQLException
     */
public void executeDDL(String pStmt)throws SQLException;

    /**
     * Выполнение Insert Update DElete
     * @param pStmt оператор SQL
     * @return количество изменённых/удалённых/вставленных строк
     * @throws SQLException
     */
public  int  executeDML(String pStmt)throws SQLException;

    /**
     * Выполнение оператора если он не вызовет зависание в блокировке
     * @param pStmt оператор SQL
     * @return список строк. В перовой - либо "locked", либо "success". Во второй либо имя блокирующего пользователя, либо количество изменённых строк
     * @throws SQLException
     */
public  ArrayList<String> executeIfNoLocks(String pStmt)throws SQLException;
    
     /**
     * Выполняет генерацию одного или более операторов для попытки 
     * блокировки данных извлекаемых оператором select. 
     * Обычно это делается добавлением фразы for update of ...
     * @param pStmt -- оператор select пример: select cl.id,cl.fullname from cl_t_client cl
     * @param uk -- список столбцов являющихся членами уникального ключа строк извлекаемых запросом
     * @return список строк - операторов
     * @throws SQLException 
     */
public ArrayList<String> select4update(String pStmt,ArrayList<ColInfo> uk)throws SQLException;
    /**
     * Сбор в строку значений первого столбца строк извлекаемых запросом
     * @param pStmt  оператор SQL 
     * @param pRows  Количество строк, которые надо извлеч из курсора:0 - Все строки;  1 - Одну строку; 2 - Две и т.д.
     * @return строка значений
     * @throws SQLException
     */

public String getSelectResult(String pStmt,int pRows)throws SQLException;

    /**
     * Получение значения 1-го столбца 1-ой строки запроса
     * @param pStmt
     * @return значение первого столбца первой строки
     * @throws SQLException
     */
public String getSelectResult(String pStmt)throws SQLException;
    
    /**
     * Получение списка значений столбцов первой строки извлекаемой запросом
     * @param pStmt
     * @return список строк - значений столбцов
     * @throws SQLException 
     */
public ArrayList<String> getSelectFirstRowValList(String pStmt)throws SQLException;

/**
     * Получение списка всех строк запроса каждый элемент списка - строка состоящая из списка значений столбцов
     * @param pStmt
     * @return список строк - значений столбцов
     * @throws SQLException 
     */
public ArrayList<ArrayList<String>> getSelectAllRowsValList(String pStmt)throws SQLException;


/**
     * Получение списка всех строк запроса каждый элемент списка - hashmap: alias/value
     * @param pStmt
     * @return список строк - значений столбцов
     * @throws SQLException 
     */
public ArrayList<LuiCollection> getSelectAllRowsValMap(String pStmt)throws SQLException;

/**
     * Получение  значений столбцов первой строки извлекаемой запросом в виде HashMap
     * @param pStmt
     * @return Map-имя столбца-&gt;значение
     * @throws SQLException 
     */
public LuiCollection getSelectFirstRowValMap(String pStmt)throws SQLException;

    /**
     * Получение всех строк 2-х столбцового запроса (ключ-значение) в виде HashMap
     * @param pStmt 
     * @return
     * @throws SQLException 
     */
public LuiCollection getSelectAllRowsTwoColsValMap(String pStmt)throws SQLException;
    /**
     * Сommit
     * @throws SQLException
     */
public void commit()throws SQLException;

    /**
     * Rollback
     * @throws SQLException
     */
public void rollback()throws SQLException;

    /**
     * Savepoint
     * @param pPointName
     * @throws SQLException
     */
public void savepoint(String pPointName)throws SQLException;

    /**
     * Rollback to Savepoint
     * @param pPointName
     * @return true -- откат к точке сохранения выполнен; false - точка сохранения не обнаружена
     * @throws SQLException
     */
public boolean rollbackToSavepoint(String pPointName)throws SQLException;

    /**
     * Проверка наличия несохранённых изменений (транзакции)
     * @return boolean true - есть, false - нет
     * @throws SQLException
     */
public boolean isTransaction()throws SQLException;

    /**
     * возвращает JDBC Connection
     * @return
     */
public  Connection getConnection();

    /**
     * Передать в сервер БД переменную
     * @param varCode - код переменной
     * @param varValue - значение переменной
     * @throws SQLException
     */
//public void varToServer(String varCode,String varValue)throws SQLException; 

    /**
     * Забрать переменную с сервера БД
     * @param varCode - код переменной
     * @return
     * @throws java.sql.SQLException
     */
//public String varFromServer(String varCode)throws SQLException;

    /**
     * Открыть курсор в данном соединении
     * @param pStmt - SQL оператор
     * @return номер курсора в соединении
     * @throws SQLException
     */
public int CursorOpen(String pStmt) throws SQLException;
    /**
     * Закрытие курсора
     * @param pCursID - номер курсора в соединении
     * @throws SQLException
     */
public void CursorClose(int pCursID) throws SQLException;
/**
 * Количество открытых курсоров
 * @return 
 */
public int CursorOpenedCount();

    /**
     * Извлечение очередной строки из курсора
     * @param pCursID - номер курсора в соединении
     * @return true - строка извлечена false - данных больше нет
     * @throws SQLException
     */
public boolean CursorNextRow(int pCursID)throws SQLException;

    /**
     * Проверка открытости курсора
     * @param pCursID - номер курсора в соединении
     * @return true - открыт, false - закрыт
     */
public boolean CursorisOpen(int pCursID);

    /**
     * Возвращает количество столбцов в строке курсора
     * @param pCursID - номер курсора в соединении
     * @return количество столбцов в строке
     */
public int CursorGetCols(int pCursID);

    /**
     * Получить описание столбцов запроса курсора
     * @param pCursID - номер курсора в соединении
     * @return список объектов ColInfo
     */
public ArrayList<ColInfo> CursorGetColsInfo(int pCursID);

     /**
     * Получить текущую строку c данными курсора (последняя извлечённая строка запроса)
     * @param pCursID - номер курсора в соединении
     * @return список объектов ColInfo
     */
public ArrayList<String> CursorGetRow(int pCursID);

    /**
     * Закрытие всех открытых курсоров
     * @throws SQLException
     */
public default void CloseAllCursors() throws SQLException { CloseAllCursors(false); }

public void CloseAllCursors(boolean pDebug) throws SQLException;

/**
     * Возвращает количество миллисекунд пршедших с 01.01.1970 до момента последней активности (т.е. к-либо действий) соединения
     */
public long getLastActivity();

    /** Установить для сеанса в БД информацию, которая будет доступна и видна в списке сеансов системных view СУБД
     * @param pInfo -- строка с описанием сеанса 
     * @throws SQLException 
     */
public void setSessionInfo(String pInfo) throws SQLException;
    
    /** Получить информацию установленную в setSessionInfo
     * @return -- строка с описанием сеанса
     */
public String getSessionInfo();

/**
 * Преобразование строки к числу/дате/времени... и обратно к строке по указанному формату
 * исходная строка может не соответствовать формату, но нужно попытаться её преобразовать
 * с использованием подмен , на .
 * @param pString - исходная строка
 * @param pFormat - формат
 * @param pLuiDataType - тип данных Lui:NUMBER DATE DATETIME
 * @return
 * @throws SQLException 
 */
//public String StringToStringByFormat(String pString,String pFormat,String pLuiDataType)throws SQLException;
/**
 * Выдаёт количество строк выфетченных последним запросом в функциях getSelect...
 * @return количество строк извлечённое последним запросом
 */
public int getRowCount();
/**
 * Выдаёт следующее значение из последовательности
 */
public String NextVal(String pSequence)throws SQLException;
/**
 * Модернизирует запрос для обращения к функции типа offset NN rows  для применения в операторе SELECT в соединениях с короткими транзакциями
 */
public String setOffsetClause(String query,String offsetColAlias,String hint,int rows_fetched);
/*
* Возвращает фразу для столбца смещения если не поддерживается offset nnn rows
* Пример rownum alias
*/
public String getOffsetColumn(String alias);
/**
 * Установка для соединения NLS-настроек
 * @param pLocale 
 */
public void setLocale(String pLocale);
/**
 * Вычисляет поставщика СУБД
 * Возможные значения:PG,ORA,MyS,MS
 * @return 
 */
public String getDBvendor();
/**
 * Формирование строки, содержащей обращение к данным в заданном столбце, для её использования в SQL-запросе
 * @param pColumn - код столбца
 * @param pLuiDataTypeSrc - исходный тип данных (LUI)
 * @param pLuiDataTypeTgt - целевой тип данных (LUI)
 * @param pLuiFormat - формат данных
 * @return 
 */
public String getColumnSQLClause (String pColumn, String pLuiDataTypeSrc, String pLuiDataTypeTgt, String pLuiFormat);
/**
 * Сохраняет файл в clob
 * @param pStmt -- оператор SQL (Insert или Update) с одной единственной bind-переменной заданной в виде знака вопроса
 * нпример:INSERT INTO table_images(data) VALUES(?)
 * @param pFile -- строка с путём и именем файла
 
     * @throws java.sql.SQLException
     * @throws java.io.IOException
 */
public void saveFileToClob(String pStmt,String pFile) throws SQLException,IOException;

/** извлекает clob в файл
 * @param pStmt -- оператор Select извлекающий одну строку (рассмотрена будет только первая) и один столбец (blob(bytea)).
 * @param pFile -- строка с путём и именем файла
 
     * @throws java.sql.SQLException
     * @throws java.io.IOException
 *
 */
public void readClobToFile(String pStmt,String pFile) throws SQLException,IOException;

/**
 * Сохраняет файл в blob
 * @param pStmt -- оператор SQL (Insert или Update) с одной единственной bind-переменной заданной в виде знака вопроса
 * нпример:INSERT INTO table_images(data) VALUES(?)
 * @param pFile --строка с путём и именем файла
  
     * @throws java.sql.SQLException
     * @throws java.io.IOException
 */
public void saveFileToBlob(String pStmt,String pFile) throws SQLException,IOException;

/** извлекает blob в файл
 * @param pStmt -- оператор Select извлекающий одну строку (рассмотрена будет только первая) и один столбец (blob(bytea)).
     * @param pFile -- строка с путём и именем файла
 
     * @throws java.sql.SQLException
     * @throws java.io.IOException
 *
 */
public void readBlobToFile(String pStmt,String pFile) throws SQLException,IOException;

/* возвращает имя схемы */
public String getSchema();

/*
*/

    /**
     * возвращает select для извлечения данных о столбцах таблицы(view)
       в запросе д.б. столбцы:
       SCHEMA- имя схемы(владельца в Oracle)
       NAME- имя столбца
       DATATYPE- тип данных СУБД
       COMMENT- комментарий
       PK- признак вхождения в перв.ключь (Y/N)
       NULLABLE- признак НЕ обязательности данных (Y/N)
     * @param pTable - имя таблицы, возможно с именем схемы и даже псевдонимом 
     * пример: lui.lui_t_user usr
     * @return - запрос
     */

public String getQuery4Column(String pTable);

    /**
     * возвращает select для извлечения данных о таблицах или view
       в запросе д.б. столбцы:
       SCHEMA- имя схемы(владельца в Oracle)
       TABLE_NAME- имя таблицы
       TABLE_TYPE- тип данных СУБД
       COMMENTS- комментарий
       
     */

public String getQuery4Table();

    
}