/*
 * Copyright (c) 2021 «FORS Telecom» LLC.
 * All Rights Reserved.
 *
 * The code is published under the terms of the «FORS Telecom» LLC Licens,
 * published in file LICENSE of this package or at
 *
 * http://lui.fors.ru/license	- in Russian
 * http://lui.fors.com/license	- in English.
 *
 */

/*
 * Вычисление показателей описательной статистики для:
 *  - столбца списка,
 *  - столбца или строки или набора выделенных ячеек матрицы.
 *
 * Результат оформляется в виде массива в формате JSON и помещается в параметр DESCRIPTIVE_STATS.
 * 
 */
package Standard;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.lang.Math;

import com.google.gson.Gson;

import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;
import org.apache.commons.math3.stat.StatUtils;

import LuiCollection.LuiCollection;
import static LuiCollection.LuiString.is_not_null;
import static LuiCollection.LuiString.is_null;
import LuiDbInterface.ColInfo;
import LuiDbInterface.LuiDbConn;
import LuiExceptionMsg.LuiException;

import luiserver.LuiAppConn;
import luiserver.LuiAppElement;
import luiserver.LuiAppForm;
import luiserver.LuiAppProp;
import luiserver.LuiApplication;
import luiserver.LuiClCommand;
import luiserver.LuiClCommandGroup;
import luiserver.LuiClCommandWriter;
import luiserver.LuiClEvent;
import luiserver.LuiClEventReader;
import luiserver.LuiPlugin;


/**
 *
 * @author vsenchenko
 */
public class DescriptiveStats implements LuiPlugin
{
    boolean print_debug = true;
    LuiApplication app;
    LuiCollection cs = null;
    LuiPlugin cellPropertyPlugin;
    String result;

    String formId;   
    //String colCode;
    //String rowCode;
    int colNo;
    int rowNo;

    Mode mode = new Mode();
    
    class SelectedCell
    {
      String row;
      String col;
    }

    class Mode
    {
        HashMap <Double, Integer> vals = new HashMap<>();
        
        public void clear()
        {
            vals.clear();
        }
        
        public void add(Double val)
        {
            if( vals.containsKey(val) )
                vals.put(val, vals.get(val) + 1);
            else
                vals.put(val, 1);
        }

        public String get_mode()
        {            
            int max_n = 0;
            
            for( Double key : vals.keySet() )
            {
                int n = vals.get(key);
                if( n > max_n ) max_n = n;
            }

            String s = "";

            if( max_n > 1 )
            {
                for( Double key : vals.keySet() )
                {
                    if( vals.get(key) == max_n )
                    {
                        s = s + Double.toString(key) + ", ";
                    }
                }
            }
            
            if(s.length() > 2) s = s.substring(1, s.length() - 2);
            
            return s;
        }
    }
    
    // SVA ---------------------------------------------------------------------

    private void DEBUG(String str)
    {
       if(print_debug) System.out.println(str);
    }

    private void DEBUGP(String str)
    {
       if(print_debug) System.out.print(str);
    }

    private void DEBUG(String format, Object... arguments)
    {
        if(print_debug) System.out.printf(format+"\n", arguments);
    }

    // SVA ---------------------------------------------------------------------

    private void DEBUG_FORM_INFO(LuiAppForm frm) throws LuiException, SQLException
    {
        if(!print_debug) return;

        // все параметры формы (для отладки)
        DEBUG("------ Form Properties ------");
        for(String k : frm.properties.keySet())
        {
            //DEBUG("frm.prop %s = %s", k, frm.properties.get(k));
            DEBUG("frm.prop %s = %s", k, frm.getProperty(k));            
        }

        DEBUG("~~~~~~ Form Properties ~~~~~~");

        // все элементы формы
        DEBUG("------ Form Elements ------");
        for( String ek : frm.elements.keySet() )
        {
            DEBUG("--->>> %s (%s)", frm.elements.get(ek).getType(), ek);                       
            LuiAppElement el = frm.elements.get(ek);            
            DEBUG("el.Code(%s) el.Type=%s", ek, el.getType());

            for( String pk : el.properties.keySet() )
            {
                LuiAppProp p = el.properties.get(pk);               
                DEBUG("    [%s] %s", pk, p.getCurrentValue());
            }
        }
        DEBUG("~~~~~~ Form Elements ~~~~~~");

        if(frm.MatrixMap == null) return;
        
        DEBUG("------ MatrixMap ------");

        for(String k : frm.MatrixMap.keySet())
        {
            String s = "";
            ArrayList<String> cellList = frm.MatrixMap.get(k);
            for(int i = 0; i < cellList.size(); i++)
            {
                s = s + cellList.get(i) + "<"+frm.elements.get(cellList.get(i)).getProperty("CELL_SHOW")+">, ";
            }

            DEBUG("-- [%s] => %s", k, s);
        }

        DEBUG("~~~~~~ MatrixMap ~~~~~~");
    }

    // SVA ---------------------------------------------------------------------

    private void jadd(int i, String stat, String val)
    {
        if(result == null)
            result = "[";
        else
            result = result.concat(",");
                
        result = result.concat("{\"seq\":\"" + i + "\",\"stat\":\"" + stat + "\",\"val\":\"" + val + "\"}");
    }

    // SVA ---------------------------------------------------------------------

    private void jadd(int i, String stat, double val)
    {
        jadd(i, stat, Double.toString(val));
    }

    // SVA ---------------------------------------------------------------------

    @Override
    public void setContext(LuiApplication pContext)
    {
        app=pContext;
    }

    // SVA ---------------------------------------------------------------------

    @Override
    public void findDependency(String pString, LuiAppProp pProp) throws LuiException, SQLException
    {
        throw new UnsupportedOperationException("Not supported yet.");
        //To change body of generated methods, choose Tools | Templates.
    }

    // SVA ---------------------------------------------------------------------

    private String getCellProperty(String cellAddr, String prop) throws LuiException, SQLException
    {
        //String val = cellPropertyPlugin.compute(cellAddr + "." + prop, null);
        //DEBUG(cellAddr + "." + prop + " = " + val);
        //return val;
        return cellPropertyPlugin.compute(cellAddr + "." + prop, null);
    }
    
    // SVA ---------------------------------------------------------------------

    private void collectDoubleValue(DescriptiveStatistics stats, String val)
    {
        DEBUG("collectValue: " + val);

        if( is_null(val) ) return;
            
        val = val.replaceAll("\\h", "").replace(",", ".").replace(" ", "");

        try
        {
            Double dval = Double.parseDouble(val);
            stats.addValue(dval);
            mode.add(dval);
        }
        catch(NumberFormatException ex)
        {
            DEBUG("-- NumberFormatException --");
        }    
    }
    
    // SVA ---------------------------------------------------------------------

    private void collectValue(DescriptiveStatistics stats, String cellAddr)  throws LuiException, SQLException
    {
        String val = getCellProperty(cellAddr, "CELL_SHOW");

        DEBUG("cellAddr: " + cellAddr);

        collectDoubleValue(stats, val);
    }

    // SVA ---------------------------------------------------------------------

    private void collectValue(DescriptiveStatistics stats, LuiAppForm frm, int x, int y)  throws LuiException, SQLException
    {
        String idx = y + ":" + x;

        DEBUG("idx = " + idx);

        ArrayList<String> cellList = frm.MatrixMap.get(idx);

        LuiAppElement el = frm.elements.get(cellList.get(cellList.size()-1));

        //collectValue(stats, formId + "." + rowCode + ":" + el.getProperty("COL"));
        collectValue(stats, formId + "." + el.getProperty("ROW") + ":" + el.getProperty("COL"));
    }

    // SVA ---------------------------------------------------------------------

    private DescriptiveStatistics collectMatrixData(LuiAppForm frm, String calc_by, String curCellCode) throws LuiException, SQLException
    {
        LuiAppElement e = frm.elements.get(curCellCode);
        
        formId = frm.getFormID();
        
        //colCode = e.getProperty("COL");
        //rowCode = e.getProperty("ROW");

        colNo = Integer.parseInt(e.getProperty("ColNo"));
        rowNo = Integer.parseInt(e.getProperty("RowNo"));
        
        int cols = Integer.parseInt(frm.getProperty("LastCOL"));
        int rows = Integer.parseInt(frm.getProperty("LastROW"));

        DescriptiveStatistics stats = new DescriptiveStatistics();
        
        if( calc_by.equals("CELLS") )
        {
            String selectedCells = frm.getProperty("SelectedCells");
            DEBUG("selectedCells = " + selectedCells);
            Gson gson = new Gson();
            SelectedCell[] selected = gson.fromJson(selectedCells, SelectedCell[].class);
            DEBUG("selected = " + gson.toJson(selected));

            for( SelectedCell sc : selected )
                collectValue(stats, formId + "." + sc.row + ":" + sc.col);
        }
        else if( calc_by.equals("COL") )
        {
            for(int y = 1; y <= rows; y++)
                collectValue(stats, frm, colNo, y);               
        }
        else // ROW
        {
            for(int x = 1; x <= cols; x++)
                collectValue(stats, frm, x, rowNo);
        }

        return stats;
    }
    
    
    // frm.prop SelectedCells = [{"row":"T1","col":"T15"},{"row":"T2","col":"T15"},{"row":"T2","col":"T16"},{"row":"T2","col":"T17"}]

    // SVA ---------------------------------------------------------------------
    
    private DescriptiveStatistics collectListData(LuiAppForm frm, String calc_by, String curCellCode) throws LuiException, SQLException
    {
        DescriptiveStatistics stats = new DescriptiveStatistics();

        LuiDbConn lc = null;

        String queryText = frm.getVal("MainQuery");
        
        if( is_not_null(queryText) )
        {
            String acc = frm.getProperty("Connection");
            if( is_not_null(acc) )
            {
                if (app.userDbConnections.containsKey(acc))
                {
                    LuiAppConn ac = app.userDbConnections.get(acc);
                    lc = null;
                    Integer cursor = null;

                    if( ac.getSessionMode() == 'C' )
                        lc = ac.connPool.GetFree(app.NLS.currentLang.locale,app.NLS.currentLang.code,app.code,app.getLuiSessionID(),app.getAppUser());
                    else
                        lc = ac.currentConnection;  

                    try
                    {
                        //queryText = "select \"" + curCellCode + "\" from (" + queryText + ") as qq";
                        //DEBUG("queryText = " + queryText);
                        
                        cursor = lc.CursorOpen(queryText);// курсор
                        //int cols = lc.CursorGetCols(cursor); 
                        ArrayList<ColInfo> colList = lc.CursorGetColsInfo(cursor);
                                
                        int qcc=0;
                        for(int iq=0; iq<frm.elements.size(); iq++)
                        {
                            LuiAppElement el = frm.elSort(iq);
                            if( !el.getType().equals("ITEM") ) continue;
                            if( el.colinfo == null ) continue;
                            colList.get(qcc).alias = el.Code;
                            qcc++;
                        }
                        
                        int colIdx=0;                        

                        //for (ColInfo ci : colList) DEBUG(">>> alias = " + ci.alias + " name = " + ci.name);
    
                        for( int i=0; i<colList.size(); i++ )
                        {
                            String colCode = colList.get(i).alias;
                            if( colCode.equals(curCellCode) )
                            {
                                colIdx = i;
                                break;
                            }
                        }
    
                        ArrayList<String> srow;

                        while( lc.CursorNextRow(cursor) )
                        {
                            srow = lc.CursorGetRow(cursor); 

                            String val = srow.get(colIdx);
                            
                            collectDoubleValue(stats, val);
                        }
                    }                    
                    //catch( Exception ex) // временно
                    //{
                    //    DEBUG("Exception: " + ex.getLocalizedMessage());
                    //}
                    finally
                    {
                       // if( lc != null )
                        //{
                            if(cursor!=null && lc.CursorisOpen(cursor) ) lc.CursorClose(cursor);
                            if( ac.getSessionMode() == 'C' )
                            {
                                ac.connPool.SetFree(lc);
                            }
                        //}
                    }
                }   
            }  
        }

        return stats;
    }
    
    // SVA ---------------------------------------------------------------------
    
    @Override
    public String compute(String pString, LuiAppProp pProp) throws LuiException, SQLException
    {
        DEBUG("compute");

        result = "";

        mode.clear();
        
        //String fc = app.getVal("FormID");
        //String fc = app.getVal("ParentFormID");
        String fc = app.forms.get(app.getVal("FormID")).getProperty("Var ParentFormID");

        DEBUG("fc = " + fc);

        LuiAppForm frm = app.forms.get(fc);

        DEBUG("frm.code = " + frm.code);
        DEBUG("form type = " + frm.getType());

        cs = new LuiCollection("CurrentCell:" + pString);
        
        String curCellCode = cs.Get_Element("CurrentCell");

        DEBUG("CurrentCell = " + curCellCode);



        //String selectedCells = frm.getProperty("SelectedCells");
        //DEBUG("selectedCells = " + selectedCells);
        //Gson gson = new Gson();
        //SelectedCell[] selected = gson.fromJson(selectedCells, SelectedCell[].class);
        //DEBUG("selected = " + gson.toJson(selected));

        
        
        if(curCellCode.equals("")) return "";
        
        cellPropertyPlugin = app.parser.plugins.get("CellProperty");
        
        String calc_by = cs.Get_Element("CALC_BY");

        DEBUG_FORM_INFO(frm);

        DescriptiveStatistics stats;

        if( frm.getType().equals("GRID") )
        {
            stats = collectMatrixData(frm, calc_by, curCellCode);
        }
        else if( frm.getType().equals("LIST") )
        {
            stats = collectListData(frm, calc_by, curCellCode);
        }
        else
        {
            return "";
        }
        
        double nonzero = 0;
        for(int i = 0; i < stats.getN(); i++) if(stats.getElement(i) != 0) nonzero++;

        //double[] mode = StatUtils.mode(stats.getValues());
        //String modeStr = "";
        //for(int i = 0; i < mode.length; i++) if(stats.getElement(i) != 0) modeStr = modeStr.concat(Double.toString(mode[i]) + (i < mode.length-1?", ":""));
       
        
        int percentile = Integer.parseInt(cs.Get_Element("PERCENTILE"));
        

        // https://ru.wikipedia.org/wiki/Описательная_статистика
        // https://commons.apache.org/proper/commons-math/javadocs/api-3.6.1/index.html
        // https://commons.apache.org/proper/commons-math/javadocs/api-3.6.1/org/apache/commons/math3/stat/descriptive/DescriptiveStatistics.html
        // https://commons.apache.org/proper/commons-math/javadocs/api-3.6.1/org/apache/commons/math3/stat/StatUtils.html
        
        /*
        DEBUG("------------------");
        DEBUG("N = " + stats.getN());
        DEBUG("max = " + stats.getMax());
        DEBUG("min = " + stats.getMin());
        DEBUG("sum = " + stats.getSum());
        DEBUG("not zero = " + notZero);       
        DEBUG("mean = " + stats.getMean());
        DEBUG("median = " + stats.getPercentile(50));
        DEBUG("StandardDeviation = " + stats.getStandardDeviation());
        DEBUG("стандартная ошибка среднего = " + stats.getStandardDeviation() / Math.sqrt(stats.getN()));
        DEBUG("коэффициент асимметрии = " + stats.getSkewness());
        DEBUG("коэффициент эксцесса = " + stats.getKurtosis());
        DEBUG("Percentile(%d) = %f", percentile, stats.getPercentile(percentile));
        DEBUG("мода = " + modeStr);
        */
                
        result = null;
        
        jadd(1, "Размер выборки", stats.getN());
        jadd(2, "Минимум", stats.getMin());
        jadd(3, "Максимум", stats.getMax());
        jadd(4, "Сумма", stats.getSum());
        jadd(5, "Не нулевых", nonzero);
        jadd(6, "Среднее арифметическое", stats.getMean());
        jadd(7, "Медиана", stats.getPercentile(50));
        jadd(8, "Стандартное отклонение", stats.getStandardDeviation());
        jadd(9, "Стандартная ошибка", stats.getStandardDeviation() / Math.sqrt(stats.getN()));
        jadd(10, "Асимметрия", stats.getSkewness());
        jadd(11, "Эксцесс", stats.getKurtosis());
        jadd(12, "Перцентиль P" + percentile, stats.getPercentile(percentile));
        //jadd(13, "Мода", modeStr);
        jadd(13, "Мода", mode.get_mode());
        
        result = result.concat("]");

        mode.clear();
        
        return result;
    }
    
    @Override
    public void execute(String pString, LuiAppProp pProp) throws LuiException, SQLException
    {
        DEBUG("pString = " + pString);

        String fc = app.getVal("FormID");
        if( is_null(fc) ) return;
        if( !app.forms.containsKey(fc) ) return;

        String r = compute(pString, pProp);

        DEBUG("result = " + r);

        app.forms.get(app.getVal("FormID")).setProperty("DESCRIPTIVE_STATS", r);
    }    
}
