/*
 * Copyright (c) 2021 «FORS Telecom» LLC.
 * All Rights Reserved.
 *
 * The code is published under the terms of the «FORS Telecom» LLC Licens,
 * published in file LICENSE of this package or at
 *
 * http://lui.fors.ru/license	- in Russian
 * http://lui.fors.com/license	- in English.
 *
 */

package ru.fors.telecom.webui.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URLDecoder;
import java.util.Arrays;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 *
 * @author geliseev
 */
public class FileServlet extends HttpServlet {
    
    private static final int BUFFER_SIZE = 16384;
    private static final long EXPIRE_TIME = 3600000; // 1 час
    private File mHomePath;
    private String mHomePathCanonical;
    
    @Override
    public void init() throws ServletException {
        String aHomePath = getInitParameter("basePath");
        if(aHomePath == null) {
            throw new ServletException("FileServlet requires initial parameter 'basePath'");
        }
        
        mHomePath = new File(aHomePath);
        if(!mHomePath.exists()) {
            throw new ServletException(String.format("FileServlet basePath='%s' does not exists", aHomePath));
        }
        if(!mHomePath.isDirectory()) {
            throw new ServletException(String.format("FileServlet basePath='%s' is not a directory", aHomePath));
        }
        if(!mHomePath.canRead()) {
            throw new ServletException(String.format("FileServlet basePath='%s' is not readable", aHomePath));
        }
        try {
            mHomePathCanonical = mHomePath.getCanonicalPath();
        }
        catch(IOException ex) {
            throw new ServletException(ex);
        }
        
    }
    
    

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        processFileRequest(req, resp);
    }
    
    @Override
    protected void doHead(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        processFileRequest(req, resp);
    }
    

    
    protected void processFileRequest(HttpServletRequest pReq, HttpServletResponse pResp) throws ServletException, IOException {
        
        String aRequestedFile = pReq.getPathInfo();
        
        if(aRequestedFile == null) { 
            pResp.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
        }
        
        File aFile = new File(mHomePath, URLDecoder.decode(aRequestedFile, "UTF-8"));
       
        if(!aFile.exists()) { pResp.sendError(HttpServletResponse.SC_NOT_FOUND); return;}
        if(!aFile.isFile()) { pResp.sendError(HttpServletResponse.SC_NOT_FOUND); return;}
        
        if(!aFile.getCanonicalPath().startsWith(mHomePathCanonical)) { 
            pResp.sendError(HttpServletResponse.SC_FORBIDDEN, "jail break out");
            return;
        }
              
        String aMimeType = getServletContext().getMimeType(aFile.getName());
        if(aMimeType == null || aMimeType.isEmpty()) {
            aMimeType = "application/octet-stream";
        }
        
        long aFileLength = aFile.length();
        long aLastModified = aFile.lastModified();
        String aETAG = String.format("%s_%x_%x", aFile.getName(), aFileLength, aLastModified);
        //long aExpires = System.currentTimeMillis() + EXPIRE_TIME;
        // Expires вредит - браузеры не хочут перегружать файл
        
        
        
        String aIfNoneMatch = pReq.getHeader("If-None-Match");
        if (aIfNoneMatch != null && matches(aIfNoneMatch, aETAG)) {
            pResp.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
            pResp.setHeader("ETag", aETAG); // Required in 304.
            //pResp.setDateHeader("Expires", aExpires); // Postpone cache with 1 week.
            return;
        }
        
        /*
        long aIfModifiedSince = pReq.getDateHeader("If-Modified-Since");
        if ( aIfModifiedSince != -1 && aIfModifiedSince + 1000 > aLastModified) {
            pResp.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
            pResp.setHeader("ETag", aETAG); 
            pResp.setDateHeader("Expires", aExpires); 
            return;
        }
        */
        
        String aIfMatch = pReq.getHeader("If-Match");
        if (aIfMatch != null && !matches(aIfMatch, aETAG)) {
            pResp.sendError(HttpServletResponse.SC_PRECONDITION_FAILED);
            return;
        }
        
        /*
        long aIfUnmodifiedSince = pReq.getDateHeader("If-Unmodified-Since");
        if (aIfUnmodifiedSince != -1 && aIfUnmodifiedSince + 1000 <= aLastModified) {
            pResp.sendError(HttpServletResponse.SC_PRECONDITION_FAILED);
            return;
        }
        */
        
        
        pResp.reset();
        pResp.setBufferSize(BUFFER_SIZE);
        pResp.setContentType(aMimeType);
        pResp.setHeader("ETag", aETAG);
        pResp.setDateHeader("Last-Modified", aLastModified);
        //pResp.setDateHeader("Expires", aExpires);
        
        // Не отдаём контент, если метод - head
        if(pReq.getMethod().equalsIgnoreCase("HEAD")) {
            return;
        }
        
        pResp.setContentLengthLong(aFile.length());
        
      
        byte[] aDataBuffer = new byte[BUFFER_SIZE];
        int aReaded;
        
        try (
            FileInputStream aIS = new FileInputStream(aFile);
            ServletOutputStream aOS = pResp.getOutputStream();
        ) {
            while((aReaded=aIS.read(aDataBuffer))>0) {
                aOS.write(aDataBuffer, 0, aReaded);
            }
        }
    }

    
    private static boolean matches(String matchHeader, String toMatch) {
        String[] matchValues = matchHeader.split("\\s*,\\s*");
        Arrays.sort(matchValues);
        return Arrays.binarySearch(matchValues, toMatch) > -1
            || Arrays.binarySearch(matchValues, "*") > -1;
    }
    
    
}
