/*
 * Copyright (c) 2021 «FORS Telecom» LLC.
 * All Rights Reserved.
 *
 * The code is published under the terms of the «FORS Telecom» LLC Licens,
 * published in file LICENSE of this package or at
 *
 * http://lui.fors.ru/license	- in Russian
 * http://lui.fors.com/license	- in English.
 *
 */

package ru.fors.telecom.webui.core;

import com.google.gson.JsonObject;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.net.Socket;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import ru.fors.telecom.webui.common.Defaults;
import ru.fors.telecom.webui.common.Defaults.Instructions;
import ru.fors.telecom.webui.common.PollResponse;
import ru.fors.telecom.webui.common.Utils;

/**
 *
 * @author john
 */
public class MessageHandler implements Runnable {
    
    private final Socket msgSocket;
    private final BufferedReader msgReader;
    private final ConcurrentLinkedQueue<JsonObject> instructions;
    private final ArrayList<HashMap<String, String>> globalParams;
    private final HashMap<String, String> localParams;
    private final HashMap<String, String> effectiveParams;
    //private final SAXBuilder xmlBuilder;
    private String commandTag;
    private int commandLevel;
    private String commandToProcess;
    private String param;
    private String msgID;
    private final PollResponse defaultResponse;

    
    public MessageHandler (Socket socket, BufferedReader reader) {
        msgSocket = socket;
        msgReader = reader;
        commandToProcess = null;
        commandLevel = -1;
        commandTag = null;
        defaultResponse = new PollResponse();
        defaultResponse.setRetry(Defaults.NORMAL_POLL_INTERVAL);
        instructions = new ConcurrentLinkedQueue<>();
        //xmlBuilder = new SAXBuilder();
        localParams = new HashMap<>();
        effectiveParams = new HashMap<>();
        globalParams = new ArrayList<>();
    }

    public void finish() {
        try {
            msgSocket.close();
        } catch (IOException ex) {}
        try {
            msgReader.close();
        } catch (IOException ex) {}
    }
    
    @Override
    public void run() {
        try {
            while (!(msgSocket.isClosed())) {
                String line;
                StringBuilder sb = new StringBuilder();
                while ((line = msgReader.readLine()) != null) {
                    if (line.equals("")) {
                        break;
                    } else {
                        sb.append(line.replaceAll(Defaults.LF_REPLACE, "\n"));
                    }
                }
                processCommands(sb.toString());
            }
        } catch (Exception ex) {} 
    }
    
    public synchronized void processCommands(String rawCommands) throws JDOMException, IOException {
        
        SAXBuilder xmlBuilder = new SAXBuilder();
        Document XMLCommands = xmlBuilder.build(new StringReader(rawCommands));
        Utils.reorderXML(XMLCommands.getRootElement());
        List<Element> commandsList = XMLCommands.getRootElement().getChildren("W");
        if (commandsList.isEmpty()) {
            buildInstructions(XMLCommands.getRootElement());
        } else {
            Iterator<Element> iter = commandsList.iterator();
            while (iter.hasNext()) {
                buildInstructions(iter.next());
            }
        }
    }
    
    private void buildInstructions(Element elem) {
        globalParams.clear();
        localParams.clear();
        commandTag = "";
        processXML(elem, 0);
        processCommand();
    }
    
    private void processXML(Element e, int level) {
        if (e.getName().equalsIgnoreCase("Command") || (e.getName().equalsIgnoreCase("W"))) {
            commandTag = "W";
            processCommand();
        } else if (e.getName().equalsIgnoreCase("C")) {
            processCommand();
            commandTag = "C";
            commandToProcess = e.getAttributeValue("N");
            commandLevel = level;
        }
        if (e.getName().equalsIgnoreCase("P")) {
            if (commandTag.equalsIgnoreCase("W")) {
                if (globalParams.size() < level) {
                    for (int i = globalParams.size() + 1; i <= level; i++) {
                        globalParams.add(new HashMap<>());
                    }
                }
                globalParams.get(level - 1).put(e.getAttributeValue("N"), e.getText());
            } else if (commandTag.equalsIgnoreCase("C")) {
                localParams.put(e.getAttributeValue("N"), e.getText());
            }
        }
        List<Element> children = e.getChildren();
        Iterator<Element> iter = children.iterator();
        while (iter.hasNext()) {
            Element child = iter.next();
            processXML(child, level + 1);
        }
    }
    
    private String checkParameter(String param) {
        if (effectiveParams.containsKey(param)) {
            return effectiveParams.get(param);
        }
        return null;
    }
    
    private void processCommand() {
        if (commandToProcess != null) {
            effectiveParams.clear();
            try {
                int i;
                for (i = 0; i < commandLevel; i++) {
                    if ((!globalParams.isEmpty()) && (!globalParams.get(i).isEmpty())) {
                        effectiveParams.putAll(globalParams.get(i));
                    }
                }
            } catch (Exception ex) {
            }
            effectiveParams.putAll(localParams);
            
            switch (commandToProcess) {
                
                case "Message":
                    msgID = "sys-" + Utils.generateId();
                    instructions.add(Utils.instruction(Instructions.msg, msgID));
                    param = checkParameter("Title");
                    if (param != null) {
                        instructions.add(Utils.instruction(Instructions.ttl, param));
                    }
                    param = checkParameter("Type");
                    String s = "fail";
                    if (!Utils.testEmptyString(param)) {
                        s = param.toLowerCase();
                    }
                    instructions.add(Utils.instruction(Instructions.type, s));
                    param = checkParameter("Text");
                    if (param != null) {
                        instructions.add(Utils.instruction(Instructions.txt, param));
                    }
                    param = checkParameter("Notes");
                    
                    if (param != null) {
                        instructions.add(Utils.instruction(Instructions.note, param));
                    }
                   
                    break;    
                      
            }
         
            localParams.clear();
            commandToProcess = null; 
        }
    }
    
    public PollResponse getResponse() {

        PollResponse pr = defaultResponse;
        if (!instructions.isEmpty()) {
            pr = new PollResponse();
            Object instruction = instructions.poll();
            do {
                pr.addCommand((JsonObject)instruction);
                instruction = instructions.poll();
            } while (instruction != null);
            pr.setRetry(Defaults.NORMAL_POLL_INTERVAL);
        }
        return pr;

    }
    
}
